const logger = require('@pubsweet/logger')
const mailService = require('pubsweet-component-mail-service')
const last = require('lodash/last')

const {
  Email,
  services,
  Fragment,
  Collection,
  Team,
  User,
} = require('pubsweet-component-helper-service')

module.exports = models => async (req, res) => {
  const { collectionId, invitationId } = req.params
  const { isAccepted, reason } = req.body

  if (!services.checkForUndefinedParams(isAccepted)) {
    res.status(400).json({ error: 'Missing parameters.' })
    logger.error('some parameters are missing')
    return
  }
  const UserModel = models.User
  const user = await UserModel.find(req.user)
  try {
    const collection = await models.Collection.find(collectionId)
    const invitation = await collection.invitations.find(
      invitation => invitation.id === invitationId,
    )
    if (invitation === undefined)
      return res.status(404).json({
        error: 'Invitation not found.',
      })
    if (invitation.hasAnswer)
      return res
        .status(400)
        .json({ error: 'Invitation has already been answered.' })
    if (invitation.userId !== user.id)
      return res.status(403).json({
        error: `User is not allowed to modify this invitation.`,
      })

    const collectionHelper = new Collection({ collection })
    const fragment = await models.Fragment.find(last(collection.fragments))
    const fragmentHelper = new Fragment({ fragment })
    const parsedFragment = await fragmentHelper.getFragmentData({
      handlingEditor: collection.handlingEditor,
    })
    const baseUrl = services.getBaseUrl(req)
    const {
      authorsList: authors,
      submittingAuthor,
    } = await collectionHelper.getAuthorData({ UserModel })
    const emailHelper = new Email({
      UserModel,
      collection,
      parsedFragment,
      baseUrl,
      authors,
    })
    const teamHelper = new Team({ TeamModel: models.Team, collectionId })
    const userHelper = new User({ UserModel })

    if (invitation.role === 'handlingEditor')
      await collectionHelper.updateHandlingEditor({ isAccepted })
    invitation.respondedOn = Date.now()
    invitation.hasAnswer = true
    const eic = await userHelper.getEditorInChief()
    const toEmail = eic.email
    if (isAccepted === true) {
      invitation.isAccepted = true
      if (
        invitation.role === 'reviewer' &&
        collection.status === 'reviewersInvited'
      )
        await collectionHelper.updateStatus({ newStatus: 'underReview' })

      await collection.save()
      try {
        if (invitation.role === 'handlingEditor')
          await mailService.sendSimpleEmail({
            toEmail,
            user,
            emailType: 'handling-editor-agreed',
            dashboardUrl: baseUrl,
            meta: {
              collectionId: collection.customId,
            },
          })
        if (invitation.role === 'reviewer')
          emailHelper.setupReviewerDecisionEmail({
            agree: true,
            timestamp: invitation.respondedOn,
            user,
            authorName: `${submittingAuthor.firstName} ${
              submittingAuthor.lastName
            }`,
          })
        return res.status(200).json(invitation)
      } catch (e) {
        logger.error(e)
        return res.status(500).json({ error: 'Email could not be sent.' })
      }
    } else {
      invitation.isAccepted = false

      if (invitation.role === 'handlingEditor')
        await teamHelper.updateHETeam({
          collection,
          role: invitation.role,
          user,
        })
      if (reason !== undefined) {
        invitation.reason = reason
      }
      await collection.save()

      try {
        if (invitation.role === 'handlingEditor') {
          await mailService.sendSimpleEmail({
            toEmail,
            user,
            emailType: 'handling-editor-declined',
            meta: {
              reason,
              collectionId: collection.customId,
            },
          })
        } else if (invitation.role === 'reviewer') {
          collectionHelper.updateStatusByNumberOfReviewers()
          emailHelper.setupReviewerDecisionEmail({
            agree: false,
            user,
          })
        }
      } catch (e) {
        logger.error(e)
        return res.status(500).json({ error: 'Email could not be sent.' })
      }
    }
    user.save()
    return res.status(200).json(invitation)
  } catch (e) {
    const notFoundError = await services.handleNotFoundError(e, 'collection')
    return res.status(notFoundError.status).json({
      error: notFoundError.message,
    })
  }
}
