const config = require('config')
const get = require('lodash/get')
const last = require('lodash/last')
const logger = require('@pubsweet/logger')
const mailService = require('pubsweet-component-mail-service')
const {
  Email,
  services,
  authsome: authsomeHelper,
  Fragment,
  Collection,
  Team,
  Invitation,
  User,
} = require('pubsweet-component-helper-service')

const configRoles = config.get('roles')

module.exports = models => async (req, res) => {
  const { email, role } = req.body

  if (!services.checkForUndefinedParams(email, role)) {
    res.status(400).json({ error: 'Email and role are required' })
    logger.error('User ID and role are missing')
    return
  }

  if (!configRoles.collection.includes(role)) {
    res.status(400).json({ error: `Role ${role} is invalid` })
    logger.error(`invitation attempted on invalid role ${role}`)
    return
  }
  const UserModel = models.User
  const reqUser = await UserModel.find(req.user)

  if (email === reqUser.email && !reqUser.admin) {
    logger.error(`${reqUser.email} tried to invite his own email`)
    return res.status(400).json({ error: 'Cannot invite yourself' })
  }

  const collectionId = get(req, 'params.collectionId')
  let collection
  try {
    collection = await models.Collection.find(collectionId)
  } catch (e) {
    const notFoundError = await services.handleNotFoundError(e, 'collection')
    return res.status(notFoundError.status).json({
      error: notFoundError.message,
    })
  }

  const authsome = authsomeHelper.getAuthsome(models)
  const target = {
    collection,
    path: req.route.path,
  }
  const canPost = await authsome.can(req.user, 'POST', target)
  if (!canPost)
    return res.status(403).json({
      error: 'Unauthorized.',
    })

  const collectionHelper = new Collection({ collection })
  const fragment = await models.Fragment.find(last(collection.fragments))
  const fragmentHelper = new Fragment({ fragment })
  const handlingEditor = collection.handlingEditor || {}
  const parsedFragment = await fragmentHelper.getFragmentData({
    handlingEditor,
  })
  const baseUrl = services.getBaseUrl(req)
  const {
    authorsList: authors,
    submittingAuthor,
  } = await collectionHelper.getAuthorData({ UserModel })
  const emailHelper = new Email({
    UserModel,
    collection,
    parsedFragment,
    baseUrl,
    authors,
  })
  const teamHelper = new Team({ TeamModel: models.Team, collectionId })
  const invitationHelper = new Invitation({ role })

  try {
    const user = await UserModel.findByEmail(email)
    await teamHelper.setupManuscriptTeam({ user, role })
    invitationHelper.userId = user.id
    let invitation = invitationHelper.getInvitation({
      invitations: collection.invitations,
    })

    let resend = false
    if (invitation !== undefined) {
      if (invitation.hasAnswer)
        return res
          .status(400)
          .json({ error: 'User has already replied to a previous invitation.' })
      invitation.invitedOn = Date.now()
      await collection.save()
      resend = true
    } else {
      invitation = await invitationHelper.setupInvitation({
        collection,
      })
    }

    try {
      if (role === 'reviewer') {
        if (collection.status === 'heAssigned')
          await collectionHelper.updateStatus({ newStatus: 'reviewersInvited' })

        await emailHelper.setupReviewerInvitationEmail({
          user,
          invitationId: invitation.id,
          timestamp: invitation.invitedOn,
          resend,
          authorName: `${submittingAuthor.firstName} ${
            submittingAuthor.lastName
          }`,
        })
      }

      if (role === 'handlingEditor') {
        invitation.invitedOn = Date.now()
        await collection.save()
        await collectionHelper.addHandlingEditor({ user, invitation })
        mailService.sendSimpleEmail({
          toEmail: user.email,
          user,
          emailType: 'assign-handling-editor',
          dashboardUrl: baseUrl,
        })
      }
      return res.status(200).json(invitation)
    } catch (e) {
      logger.error(e)
      return res.status(500).json({ error: 'Email could not be sent.' })
    }
  } catch (e) {
    const userHelper = new User({ UserModel })
    if (role === 'reviewer') {
      const newUser = await userHelper.setupNewUser({
        url: baseUrl,
        role,
        invitationType: 'invite',
        body: req.body,
      })
      if (newUser.error !== undefined) {
        return res.status(newUser.status).json({
          error: newUser.message,
        })
      }
      if (collection.status === 'heAssigned')
        await collectionHelper.updateStatus({ newStatus: 'reviewersInvited' })
      await teamHelper.setupManuscriptTeam({ user: newUser, role })
      invitationHelper.userId = newUser.id
      const invitation = await invitationHelper.setupInvitation({
        collection,
      })

      await emailHelper.setupReviewerInvitationEmail({
        user: newUser,
        invitationId: invitation.id,
        timestamp: invitation.invitedOn,
        authorName: `${submittingAuthor.firstName} ${
          submittingAuthor.lastName
        }`,
      })
      return res.status(200).json(invitation)
    }
    const notFoundError = await services.handleNotFoundError(e, 'user')
    return res.status(notFoundError.status).json({
      error: notFoundError.message,
    })
  }
}
